<?php
/**
 * @name inventory.php
 * @description Fichier principal du plugin d'inventaire
 * @author Benjamin Piesyk <b.pieyk@esia-sa.com>
 */
$path = $_SERVER['DOCUMENT_ROOT'];
// Inclusion des librairies
include_once $path . '/common.php';
include_once 'inventory.conf.php';
include_once MODEL_INVENTORY;
include_once LIB_INVENTORY;

$arr = Model_esia::getWebPlugin($dbconnection,false,'WEBP_INVENTORY');
$plugin_url = $arr[0]['apache_path'];
$smarty->assign('plugin_url',$plugin_url);
$page_url = "$plugin_url".$arr[0]['index_page'];
$template_path= $path.$plugin_url.'templates/';
$smarty->assign('page_url',$page_url);


$action = '';
if(isset($_GET['action']) && !empty($_GET['action']))
{
	$action = $_GET['action'];
}
$smarty->assign('action', $action);

session_write_close();

try
{
	// Filter by group ID from menu
	$groupId = 0;
	$filterDevicesList = NULL;
	$filterDevicesListEncoded = "";
	if(!empty($_REQUEST['group']) && is_numeric($_REQUEST['group']))
	{
		$groupId = intval($_REQUEST['group']);
		// Get group & sub groups children devices to filter all requests
		$filterDevicesList = Model_inventory::getInventoryDevicesIdByGroup($dbconnection, $user->getId(), $groupId);
		$filterDevicesListEncoded = json_encode($filterDevicesList);
	}
	$smarty->assign('groupId', $groupId);
	$smarty->assign('filterDevicesList', $filterDevicesListEncoded);

	if($action == '')
	{
		// Only on page load not on tab switch
		// Create navbar & page header
		$navbar->addElement($lang['INVENTORY'], $page_url);

		$invHeadBar = Model_inventory::getInventoryBuiltTypesCount($dbconnection, $user->getId(), $filterDevicesList);
		if($groupId > 0)
		{
			// Header & navbar for specific group children
			// Get current group data
			$groupInfos = Model_inventory::getGroupById($dbconnection, $groupId);
			// Add parent groups to navbar
			addNavNodeGroups($dbconnection, $navbar, $page_url, $groupId);

			$invHeadBar['nbr_nodes'] = count($filterDevicesList);
			$invHeadBar['group'] = '-';
			$invHeadBar['desc'] = '-';
			if(!empty($groupInfos['device_group_name']))
			{
				// Add current group to navbar
				$navbar->addElement($groupInfos['device_group_name'], $page_url . '?group=' . $groupId);
				// Add group name to header bar
				$invHeadBar['group'] = $groupInfos['device_group_name'];
			}
			if(!empty($groupInfos['device_group_description']))
			{
				$invHeadBar['desc'] = $groupInfos['device_group_description'];
			}
		}
		else
		{
			// Header root, all inventory displayed
			$invHeadBar['nbr_nodes'] = Model_inventory::getUserAccessDevicesNumber($dbconnection, $user->getId());
			$invHeadBar['group'] = $lang['GROUPNODE_ROOT'];
			$invHeadBar['desc'] = $lang['INVENTORY_ROOT_DESC'];
		}
		$smarty->assign('invHeadBar', $invHeadBar);
	}
	elseif($action == "main")
	{
		// Donut Fabricant
        $listCountManufacturer = Model_inventory::getCountManufacturer($dbconnection, $filterDevicesList);
		$donutData = array();
		if(empty($listCountManufacturer))
		{
			$donutData = $lang['INVENTORY_TOP_MANUFACTURER_EMPTY'];
		}
		else
		{
			foreach($listCountManufacturer as $manufacturer)
			{
				$donutData[] = array($manufacturer['name'], $manufacturer['count']);
			}
		}
		$smarty->assign('listCountManufacturer', json_encode($donutData));

		// Liste OS / Version
		$listOperatingSystem = Model_inventory::getCountOS($dbconnection, $filterDevicesList);
		$donutData = array();
		if(empty($listOperatingSystem))
		{
			$donutData = $lang['INVENTORY_TOP_OS_EMPTY'];
		}
		else
		{
			foreach($listOperatingSystem as $os)
			{
				$donutData[] = array($os['name'], $os['count']);
			}
		}
		$smarty->assign('listOS', json_encode($donutData));

		// Liste Logiciel
		$listSoftVersion = Model_inventory::getListSoftwareByVersion($dbconnection, $filterDevicesList);
		$smarty->assign('listSoftwareVersion', $listSoftVersion);

		// List Chassis
		$listChassis = Model_inventory::getListChassis($dbconnection, $filterDevicesList);
		$smarty->assign('listChassis', $listChassis);

	}
	elseif($action == "software")
	{
		// Rules state donnut
		$rulesErrorCount = Model_inventory::getCountRulesErrorByInventoryType($dbconnection, 'SOFTWARE', $filterDevicesList);
		$rulesMatch = array(
			array($lang['LGD_OK'], $rulesErrorCount['built_rule_match_count_ok']),
			array($lang['ERROR'], $rulesErrorCount['built_rule_match_count_outdated']),
		);
		$smarty->assign('rulesMatch', json_encode($rulesMatch));
		
		// Rules error table data
		$rulesTopErr = array();
		$rulesMatchState = 'errors';
		// Check if rules match
		if(!array_filter($rulesErrorCount))
		{
			// All array keys empty. No rules for software
			$rulesMatchState = 'none';
		}
		elseif(empty($rulesErrorCount['built_rule_match_count_outdated']))
		{
			// No rules in error for software
			$rulesMatchState = 'ok';
		}
		else
		{
			// Get top 5 software rules data
			$rulesTopErr = Model_inventory::getRulesInErrorByInventoryType($dbconnection, 'SOFTWARE', 5, $filterDevicesList);
		}
		$smarty->assign('rulesTopErr', $rulesTopErr);
		$smarty->assign('rulesMatchState', $rulesMatchState);
		
		// Manufacturer top 5 bars
		$topManData = formatCountManufacturerByInventoryType($dbconnection, $lang['INVENTORY_SOFTWARES'], 'SOFTWARE', 5, $filterDevicesList);
		$smarty->assign('topManData', json_encode($topManData));
		
		//$listSoftware = Model_inventory::getListInventory($dbconnection,"SOFTWARE");
		$smarty->assign('listSoftware',$listSoftware);
	}
	elseif($action == "hardware")
	{
		// Rules state donnut
		$rulesErrorCount = Model_inventory::getCountRulesErrorByInventoryType($dbconnection, 'HARDWARE', $filterDevicesList);
		$rulesMatch = array(
			array($lang['LGD_OK'], $rulesErrorCount['built_rule_match_count_ok']),
			array($lang['ERROR'], $rulesErrorCount['built_rule_match_count_outdated']),
		);
		$smarty->assign('rulesMatch', json_encode($rulesMatch));
		
		// Rules error table data
		$rulesTopErr = array();
		$rulesMatchState = 'errors';
		// Check if rules match
		if(!array_filter($rulesErrorCount))
		{
			// All array keys empty. No rules for hardware
			$rulesMatchState = 'none';
		}
		elseif(empty($rulesErrorCount['built_rule_match_count_outdated']))
		{
			// No rules in error for hardware
			$rulesMatchState = 'ok';
		}
		else
		{
			// Get top 5 hardware rules data
			$rulesTopErr = Model_inventory::getRulesInErrorByInventoryType($dbconnection, 'HARDWARE', 5, $filterDevicesList);
		}
		$smarty->assign('rulesTopErr', $rulesTopErr);
		$smarty->assign('rulesMatchState', $rulesMatchState);
		
		// Manufacturer top 5 bars
		$topManData = formatCountManufacturerByInventoryType($dbconnection, $lang['INVENTORY_HARDWARES'], 'HARDWARE', 5, $filterDevicesList);
		$smarty->assign('topManData', json_encode($topManData));
		
		// $listHardware = Model_inventory::getListInventory($dbconnection,'HARDWARE');
		$smarty->assign('listHardware',$listHardware);
	}
	elseif($action == "rules")
	{
		// Rules config
		// Interval list
		$intervalList = array();
		foreach(INV_SCAN_VALID_INTERVALS as $interval)
		{
			$intervalList[] = array('value' => $interval, 'text' => $lang['INVENTORY_SCAN_INTERVAL_' . strtoupper($interval)]);
		}
		$smarty->assign('intervalList', $intervalList);
		// Min & max date values
		$thresholdDate = new DateTime();
		$smarty->assign('minDate', $thresholdDate->format('Y-m-d'));
		$thresholdDate->modify('+2 year');
		$smarty->assign('maxDate', $thresholdDate->format('Y-m-d'));
		
		// Get previously saved config in /data directory
		$rulesConf = getInvDataJsonFile($path, INV_RULES_CONF_FILE, $lang['INVENTORY_ERROR_OPEN_FILE']);
		// Check file parsed values
		if(!isset($rulesConf['interval']) || !in_array($rulesConf['interval'], INV_SCAN_VALID_INTERVALS))
		{
			$rulesConf['interval'] =  INV_SCAN_VALID_INTERVALS[0];
		}
		$smarty->assign('intervalRules', $rulesConf['interval']);
		
		// Format dates for inputs
		$rulesNextDate = new DateTime();
		if(isset($rulesConf['next']) && is_int($rulesConf['next']))
		{
			$rulesNextDate->setTimestamp($rulesConf['next']);
		}
		else
		{
			$rulesNextDate->modify('+1 hour');
		}
		$rulesNextExec = explode("*", $rulesNextDate->format('Y-m-d*H:i'));
		$smarty->assign('rulesNextExecDate', $rulesNextExec[0]);
		$smarty->assign('rulesNextExecTime', $rulesNextExec[1]);
		
		// Table
		$listRules = Model_inventory::getListRules($dbconnection);
		$errorsCount = Model_inventory::getCountRulesErrorByRules($dbconnection);
		foreach($listRules as $rule)
		{
			// Count target devices & groups
			$countTarget = array('devices' => 0, 'groups' => 0);
			if(isset($rule['target']['devices']))
			{
				$countTarget['devices'] = count($rule['target']['devices']);
			}
			if(isset($rule['target']['groups']))
			{
				$countTarget['groups'] = count($rule['target']['groups']);
			}
			$listRules[$rule['rule_id']]['count'] = $countTarget;
			
			// Add errors count
			if(isset($errorsCount[$rule['rule_id']]))
			{
				$listRules[$rule['rule_id']]['errors'] = $errorsCount[$rule['rule_id']];
			}
			else
			{
				$listRules[$rule['rule_id']]['errors'] = array('ok' => 0, 'outdated' => 0);
			}
		}

		$smarty->assign('listRules', $listRules);
	}
	elseif($action == "importation")
	{
		// get type scan
		$typeScan = array("SNMP","Intunes","Powershell");
		$smarty->assign('typeScan',$typeScan);
		// Interval list
		$intervalList = array();
		foreach(INV_SCAN_VALID_INTERVALS as $interval)
		{
			$intervalList[] = array('value' => $interval, 'text' => $lang['INVENTORY_SCAN_INTERVAL_' . strtoupper($interval)]);
		}
		$smarty->assign('intervalList', $intervalList);
		// Min & max date values
		$thresholdDate = new DateTime();
		$smarty->assign('minDate', $thresholdDate->format('Y-m-d'));
		$thresholdDate->modify('+2 year');
		$smarty->assign('maxDate', $thresholdDate->format('Y-m-d'));
		// Get node group list
		$groupList = Model_esia::getGroupDeviceByUser($dbconnection, $user->getId());
		$smarty->assign('groupList', $groupList);
		
		$importNextDate = new DateTime('+1 hour');
		$importNextExec = explode("*", $importNextDate->format('Y-m-d*H:i'));
		$smarty->assign('importNextExecDate', $importNextExec[0]);
		$smarty->assign('importNextExecTime', $importNextExec[1]);
	}
}
catch(EsiaDbException | Exception $e)
{
	$smarty->assign('errors', $e->getMessage());
}

$smarty->display($template_path.'control_inventory.html');


// Format manufacturer data for c3 bars chart
function formatCountManufacturerByInventoryType($dbconnection, $chartLabel='', $typeInventory='ALL', $limit=5, $filterDevicesList=NULL)
{
	$manufacturerList = Model_inventory::getCountManufacturerByInventoryType($dbconnection, $typeInventory, $limit, NULL, $filterDevicesList);
	$topManData = array(
		'label' => $chartLabel,
		'values' => array(),
	);
	$manufacturerErrors = array();
	foreach($manufacturerList as $manufacturer)
	{
		$topManData['values'][] = array($manufacturer['name'], $manufacturer['count']);
	}
	
	return $topManData;
}

function addNavNodeGroups($dbconnection, $nav, $base_url, $groupId)
{
	// Add parents groups from root to current
	$arr = searchParentNode($dbconnection, $groupId);
	//on commence par la fin du tableau afin de ne pas inverser l'ordre
	$i = count($arr) - 1;
	while($i >= 0)
	{
		$nav->addElement($arr[$i]['name'], $base_url . '?group=' . $arr[$i]['id']);
		$i--;
	}
}

function searchParentNode($dbconnection, $nodegroup, $arr=array())
{
	$gr = Model_esia::getParrentGroup($dbconnection, $nodegroup);
	if(!empty($gr))
	{
		$arr[] = array(
			'id' => $gr[0]['id'],
			'name' => $gr[0]['name'],
		);
		$arr = searchParentNode($dbconnection, $gr[0]['id'], $arr);
	}
	return $arr;
}

?>
