<?php
/*
** @name: inventory.lib.php
**
** @description: functions usefull for any controllers
** @author: Quentin George <q.george@esia-sa.com>
** @date: 15/03/2023
*/

/**
*** @name: checkInvDataDir
*** @desc: Check if destination files dir is present or create it
*** @param string $path => $_SERVER['DOCUMENT_ROOT']
*** @param string $errorMsg => error message to throw in the exception if the directory cannot be created
*** @throws Exception if the directory cannot be created
**/
function checkInvDataDir($path, $errorMsg='')
{
	// Check if destination files dir is present or create it
	if(!is_dir($path . INV_DATA_PATH))
	{
		// Create data inventory folder. TRUE => recursive
		$isDir = mkdir($path . INV_DATA_PATH, 0755, TRUE);
		if(!$isDir)
		{
			throw new Exception($errorMsg . " (" . $path . INV_DATA_PATH . ")\n");
		}
	}
}
/**
*** @name: getInvDataJsonFile
*** @desc: Get inventory /data directory json file content
*** @param string $path => $_SERVER['DOCUMENT_ROOT']
*** @param string $file => file name
*** @param string $errorMsg => error message to throw in the exception if the file cannot be read
*** @return array file content as associative array
*** @throws Exception if the file exist but cannot be read
**/
function getInvDataJsonFile($path, $file, $errorMsg='')
{
	$data = array();
	if(file_exists($path . INV_DATA_PATH . $file))
	{
		$json = file_get_contents($path . INV_DATA_PATH . $file);
		$data = json_decode($json, TRUE);
		if(!is_array($data))
		{
			// json_decode return TRUE, FALSE or NULL. The file exist but cannot be opened
			throw new Exception($errorMsg . " (" . $path . INV_DATA_PATH . $file . ")\n");
		}
	}
	
	return $data;
}
/**
*** @name: setInvDataJsonFile
*** @desc: Set inventory /data directory json content into file
*** @param string $path => $_SERVER['DOCUMENT_ROOT']
*** @param string $file => file name
*** @param array $data => data to be json encoded as file content
*** @param array $errorsMsg => error messages to throw in the exceptions if the file cannot be opened and written
*** @throws Exception if the file cannot be opened or written
**/
function setInvDataJsonFile($path, $file, $data, $errorsMsg=array('', ''))
{
	$handle = fopen($path . INV_DATA_PATH . $file, "w+t");
	if(!$handle)
	{
		throw new Exception($errorsMsg[0] . " (" . $path . INV_DATA_PATH . $file . ")\n");
	}
	$isWritten = fwrite($handle, json_encode($data));
	if($isWritten == FALSE)
	{
		throw new Exception($errorsMsg[0] . " (" . $path . INV_DATA_PATH . $file . ")\n");
	}
	fclose($handle);
}

?>
