<?php

/**
 * @name Model_TableSort
 * @description Classe modèle d'Esia
 *		Toutes les méthodes sont déclarée en 'static' ainsi on garde les Objets PDO pour les programmeurs extérieurs et pour faire des modifications, ils ne sont pas /
 *		obligé de connaitre la hierachie de classe d'ESIA
 * @author Nicolas Biersart <n.biersart@esia-network.com>
 * @contributor
 * @copyright Nicolas Biersart 2012
 * @license
 * @esia 1.00
 * @version 1.0.0
 * @date 16/11/2012
 */
$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path.'/model_tablesort.php';

class Model_Inventory_TableSort extends Model_TableSort
{
	/**
	 *  retourne le tableau de correspondance entre le paramètre "cols" du tablesorter et les champs de la base de donnée et le type de tri. Il retourne uniquemejnt les colonnes pour les erreurs active
	 * @param array $lang_error_type tableau associatif faisant la correspondance entre le nom technique des erreurs (error_type) et les variables de langue.
	 * 		format FR: Array ( [warning] => Alerte,    [critical] => Critique,    [unknown] => Inconnu,    [maintenance] => Maintenance)
	 * @param array $lang_plugin tableau associatif faisant la correspondance entre le nom technique des plugins et le nom traduit
	 * 		format FR: Array ( [CHECK_SNMP_LOAD] => Processeur)
	 * @retval array  Tableau de correspondance entre le SQL et le tableSorter
	 */
	private static function ts_getListSoftware_columns()
	{
		//l'index de la column doit être égale à l'index dans le table sorter
		$columns[0]=array('col'=>'g1.device_group_name','type'=>'string');
		$columns[1]=array('col'=>'device_name','type'=>'string');
		$columns[2]=array('col'=>'device_type','type'=>'string');
		$columns[3]=array('col'=>'manufacturer_name','type'=>'string');
		$columns[4]=array('col'=>'component_name','type'=>'string');
		$columns[5]=array('col'=>'built_version','type'=>'string');
		$columns[6]=array('col'=>'built_rule_match_worst_status','type'=>'string');
		$columns[7]=array('col'=>'type_name','type'=>'string');
		$columns[8]=array('col'=>'component_description','type'=>'string');
		return $columns;
	}
	/**
	 * Récupère les erreurs en cours dans la base de donnée
	 * @param integer $user identifiant utilisateur
 	 * @param integer $typeInventory type d'inventaire
 	 * @param integer $deviceID id du device 
	 * @param array $TSParams tableau associatif envoyé par le tablesorter (format: Array(
	 *		    [filters] => Array // numéros de colonne et le filtre appliqué
	 *		        (
	 *					[4] => critique
	 *			    ),
	 *			[order] => Array // numéros de colonne qui trie en ascendant ou descendant
	 *			        (
	 *			            [8] => 1
	 *			        )
	 *			[size] => 25, //Nombre d'élement afficher dans la page
	 *			[paging] => 0) //Numéros de page
	 * @param array $lang_error_type tableau associatif faisant la correspondance entre le nom technique des erreurs (error_type) et les variables de langue.
	 * 		format FR: Array ( [warning] => Alerte,    [critical] => Critique,    [unknown] => Inconnu,    [maintenance] => Maintenance)
	 * @param array $lang_plugin tableau associatif faisant la correspondance entre le nom technique des plugins et le nom traduit
	 * 		format FR: Array ( [CHECK_SNMP_LOAD] => Processeur)
	 * @throws EsiaDbException
	 * @retval array array('total'=>$total,'rows'=>$arr); Total est le nombre d'erreur présente et le rows sont les données en tenant compte de la pagination
	 */	
	public static function ts_getListInventory($dbconnection,$user=NULL,$typeInventory=NULL,$device=NULL,$TSParams=NULL )
	{
		$columns = Model_Inventory_TableSort::ts_getListSoftware_columns();
		$param = array();
		$sql = " FROM  esia_device
				JOIN inventory_is_built ON esia_device.device_id = inventory_is_built.built_device_id
				JOIN inventory_component ON inventory_component.component_id = inventory_is_built.built_hardware_id
				JOIN inventory_type ON inventory_type.type_id = inventory_component.component_type_id
				JOIN inventory_manufacturer ON inventory_manufacturer.manufacturer_id = inventory_component.component_manufacturer_id
                LEFT JOIN esia_device_group_in ON device_id = group_in_device_id
                LEFT JOIN esia_device_group as g1 ON device_group_id=group_in_device_group_id ";
		$sql_order = "";
		$flag = Model_TableSort::ts_getActiveErrors_filter($columns,$TSParams,$sql,$param,$sql_order);
		if($flag === FALSE)
		{
			//il n'y a pas de modification effectuée et donc, on met la valeur par défaut;
			$sql_order = " ORDER BY device_name ASC";
		}
		if(!preg_match('#WHERE#',$sql))
		{
			$sql .= " WHERE ";
		}
		else
		{
			$sql .= " AND ";
		}
		if($typeInventory != "ALL")
		{
			$sql .= " inventory_type.type_type = :type AND ";
			$param[':type'] = $typeInventory;
		}
		if(!is_null($device))
		{
			if(is_numeric($device))
			{
				$sql .= " esia_device.device_id = :deviceid AND ";
				$param[':deviceid'] = $device;
			}
			elseif(is_array($device))
			{
				$in_sql = Model_esia::pdo_array_to_SQL('device_id', $device, $param);
				$sql .= " esia_device.device_id IN ($in_sql) AND ";
			}
		}
		$sql.= "device_id IN ( SELECT * FROM getUserAccessNode(:user)) ";
		$param[':user'] = $user;
		$total = 0;
		//print_r($param);
		//echo "SELECT * $sql <br /> $sql_order"; exit (0);
		
		$queryCount = $dbconnection->prepare('SELECT count(*) as nbr '.$sql);
		$queryCount = Model_esia::executeStatement($queryCount,$param);
		$row = $queryCount->fetch(PDO::FETCH_OBJ);
		$total = $row->nbr;

		$query = $dbconnection->prepare('SELECT * '.$sql.' '.$sql_order);
		
		$query = Model_esia::executeStatement($query,$param);
		$nodetype = array();
		$arr = Model_esia::getWebPluginByNodeType($dbconnection);
		foreach($arr as $row)
		{
			$nodetype[$row['type']] = $row['apache_path'].$row['control_page'];
		}
		$components=array();
		$listid = array();
		while($row = $query->fetch(PDO::FETCH_OBJ))
		{
			if(!array_key_exists($row->built_id, $components))
			{
				if(array_key_exists($row->device_type,$nodetype))
				{
					$page_control = Model_esia::replacePageControlVariable($nodetype[$row->device_type],$row);
				}
				else
				{
					$page_control = '/node_view.php?nodeid='.$row->device_id;
				}
				if(!in_array($row->device_id,$listid))
				{
					$listid[]=$row->device_id;
				}
				$components[$row->built_id] = array(
					'groupid'=> $row->device_group_id,
					'group'=> $row->device_group_name,
					'device_type'=>$row->device_type,
					'device_id' => $row->device_id,
					'device' => $row->device_name,
					'component' => $row->component_name,
					'version' => $row->built_version,
					'description' => $row->component_description,
					'type' => $row->type_name,
					'page_control'=>$page_control,
                    'state'=>$row->built_rule_match_worst_status,
                    'rule'=>$row->built_rule_match_rule_id,
					'manufacturer' => $row->manufacturer_name
				);
			}
		}
		
		return array('total'=>$total,'rows'=>$components,'listid'=>$listid);
	}
}
?>
